// Copyright 2015 The Go Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

// This file encapsulates some of the odd characteristics of the
// LARCH64 (LARCH64) instruction set, to minimize its interaction
// with the core of the assembler.

package arch

import (
	"cmd/internal/obj"
	"cmd/internal/obj/larch64"
)

func jumpLARCH64(word string) bool {
	switch word {
	case "BEQ", "BFPF", "BFPT", "BLTZ", "BGEZ", "BLEZ", "BGTZ", "BLT", "BLTU", "JIRL", "BNE", "BGE", "BGEU", "JMP", "JAL", "BL", "CALL":
		return true
	}
	return false
}

// IsLARCH64CMP reports whether the op (as defined by an larch64.A* constant) is
// one of the CMP instructions that require special handling.
func IsLARCH64CMP(op obj.As) bool {
	switch op {
	case larch64.ACMPEQF, larch64.ACMPEQD, larch64.ACMPGEF, larch64.ACMPGED,
		larch64.ACMPGTF, larch64.ACMPGTD:
		return true
	}
	return false
}

// IsLARCH64MUL reports whether the op (as defined by an larch64.A* constant) is
// one of the MUL/DIV/REM/MADD/MSUB instructions that require special handling.
func IsLARCH64MUL(op obj.As) bool {
	switch op {
	case larch64.AMUL, larch64.AMULU, larch64.AMULV, larch64.AMULVU,
		larch64.ADIV, larch64.ADIVU, larch64.ADIVV, larch64.ADIVVU,
		larch64.AREM, larch64.AREMU, larch64.AREMV, larch64.AREMVU:
		return true
	}
	return false
}

func larch64RegisterNumber(name string, n int16) (int16, bool) {
	switch name {
	case "F":
		if 0 <= n && n <= 31 {
			return larch64.REG_F0 + n, true
		}
	case "FCSR":
		if 0 <= n && n <= 31 {
			return larch64.REG_FCSR0 + n, true
		}
	case "FCC":
		if 0 <= n && n <= 31 {
			return larch64.REG_FCC0 + n, true
		}
	case "R":
		if 0 <= n && n <= 31 {
			return larch64.REG_R0 + n, true
		}
		//MSA
		//	case "W":
		//		if 0 <= n && n <= 31 {
		//			return larch64.REG_W0 + n, true
		//		}
	}
	return 0, false
}
